@@ -19,13 +19,17 @@ def get_desc(docstr):
 
     i = docstr.find("\n")
     if i != -1:
-        desc = docstr[i+2:]
+        desc = docstr[i + 2:]
     else:
         desc = "    %s" % shortdesc
     return (shortdesc, desc)
 
 def get_opts(opts):
-    for shortopt, longopt, default, desc in opts:
+    for opt in opts:
+        if len(opt) == 5:
+            shortopt, longopt, default, desc, optlabel = opt
+        else:
+            shortopt, longopt, default, desc = opt
         allopts = []
         if shortopt:
             allopts.append("-%s" % shortopt)
@@ -76,7 +80,8 @@ def show_doc(ui):
     cmds.sort()
 
     for f in cmds:
-        if f.startswith("debug"): continue
+        if f.startswith("debug"):
+            continue
         d = get_cmd(h[f])
         # synopsis
         ui.write(".. _%s:\n\n" % d['cmd'])
@@ -105,7 +110,7 @@ def show_doc(ui):
             ui.write(".. _%s:\n" % name)
         ui.write("\n")
         section(sec)
-        if callable(doc):
+        if hasattr(doc, '__call__'):
             doc = doc()
         ui.write(doc)
         ui.write("\n")
