@@ -7,24 +7,27 @@
 
 '''discover and advertise repositories on the local network
 
-Zeroconf enabled repositories will be announced in a network without
+Zeroconf-enabled repositories will be announced in a network without
 the need to configure a server or a service. They can be discovered
 without knowing their actual IP address.
 
-To allow other people to discover your repository using run "hg serve"
-in your repository::
+To allow other people to discover your repository using run
+:hg:`serve` in your repository::
 
   $ cd test
   $ hg serve
 
-You can discover zeroconf enabled repositories by running "hg paths"::
+You can discover Zeroconf-enabled repositories by running
+:hg:`paths`::
 
   $ hg paths
   zc-test = http://example.com:8000/test
 '''
 
-import Zeroconf, socket, time, os
-from mercurial import ui
+import socket, time, os
+
+import Zeroconf
+from mercurial import ui, hg, encoding
 from mercurial import extensions
 from mercurial.hgweb import hgweb_mod
 from mercurial.hgweb import hgwebdir_mod
@@ -49,7 +52,7 @@ def getip():
         dumbip = socket.gethostbyaddr(socket.gethostname())[2][0]
         if not dumbip.startswith('127.') and ':' not in dumbip:
             return dumbip
-    except socket.gaierror:
+    except (socket.gaierror, socket.herror):
         dumbip = '127.0.0.1'
 
     # works elsewhere, but actually sends a packet
@@ -98,16 +101,17 @@ def publish(name, desc, path, port):
     server.registerService(svc)
 
 class hgwebzc(hgweb_mod.hgweb):
-    def __init__(self, repo, name=None):
-        super(hgwebzc, self).__init__(repo, name)
-        name = self.reponame or os.path.basename(repo.root)
+    def __init__(self, repo, name=None, baseui=None):
+        super(hgwebzc, self).__init__(repo, name=name, baseui=baseui)
+        name = self.reponame or os.path.basename(self.repo.root)
         path = self.repo.ui.config("web", "prefix", "").strip('/')
         desc = self.repo.ui.config("web", "description", name)
-        publish(name, desc, path, int(repo.ui.config("web", "port", 8000)))
+        publish(name, desc, path,
+                int(self.repo.ui.config("web", "port", 8000)))
 
 class hgwebdirzc(hgwebdir_mod.hgwebdir):
     def __init__(self, conf, baseui=None):
-        super(hgwebdirzc, self).__init__(conf, baseui)
+        super(hgwebdirzc, self).__init__(conf, baseui=baseui)
         prefix = self.ui.config("web", "prefix", "").strip('/') + '/'
         for repo, path in self.repos:
             u = self.ui.copy()
@@ -156,7 +160,14 @@ def configitems(orig, self, section, untrusted=False):
         repos += getzcpaths()
     return repos
 
+def defaultdest(orig, source):
+    for name, path in getzcpaths():
+        if path == source:
+            return name.encode(encoding.encoding)
+    return orig(source)
+
 extensions.wrapfunction(ui.ui, 'config', config)
 extensions.wrapfunction(ui.ui, 'configitems', configitems)
+extensions.wrapfunction(hg, 'defaultdest', defaultdest)
 hgweb_mod.hgweb = hgwebzc
 hgwebdir_mod.hgwebdir = hgwebdirzc
