@@ -121,9 +121,11 @@ class ui(object):
         self._trustusers.update(self.configlist('trusted', 'users'))
         self._trustgroups.update(self.configlist('trusted', 'groups'))
 
-    def setconfig(self, section, name, value):
-        for cfg in (self._ocfg, self._tcfg, self._ucfg):
-            cfg.set(section, name, value)
+    def setconfig(self, section, name, value, overlay=True):
+        if overlay:
+            self._ocfg.set(section, name, value)
+        self._tcfg.set(section, name, value)
+        self._ucfg.set(section, name, value)
         self.fixconfig()
 
     def _data(self, untrusted):
